/*
* Copyright (c) 2008, Dmitrij Lagutin, HIIT, <dmitrij.lagutin@hiit.fi>
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License version 2 as
* published by the Free Software Foundation.
*
* Alternatively, this software may be distributed under the terms of BSD
* license.
*
* See LICENSE and COPYING for more details.
*/
#include <stdlib.h>

#include "libpla.h"
#include "libpla_io.h"
#include "../../psirpd/psirp_debug.h"


#define COUNT 200

int main(void)
{
	struct pla_hdr *pla_header;
    pla_certificate_list_item_t *pla_cert, *pla_cert2;
	unsigned char payload[10] = "1234567890";
	unsigned char psirp_header[10] = "abcdefghij";
	unsigned char *data;
	uint8_t ret;
    uint8_t rid[PSIRP_RID_LEN+1], rid2[PSIRP_RID_LEN+1];
    uint8_t sid[PSIRP_SID_LEN+1], sid2[PSIRP_SID_LEN+1];
    uint8_t *pk;
    uint32_t tmp;


    psirp_debug_init_print(PSIRP_DBG_INFO | PSIRP_DBG_ERR | PSIRP_DBG_WARN, 0);

    PSIRP_DEBUG(PSIRP_DBG_INFO, "Calling libpla_init\n");
    libpla_init(NULL, PLA_CRYPTO_SW); /* No rid present */

    //libpla_init("/home/psirp/rid.cert", PLA_CRYPTO_SW); /* Rid certificate */
    //libpla_init(NULL, PLA_CRYPTO_NULL);

    /* Create a new PLA header */
    pla_header = malloc(sizeof(struct pla_hdr));
    pla_cert = malloc(sizeof(pla_certificate_list_item_t));
    pla_cert2 = malloc(sizeof(pla_certificate_list_item_t));
    data = malloc(sizeof(struct pla_hdr)+10+10);


    /* Read rid & sids */
    libpla_io_string_hexread("1234567890a0a1a2a3a4a5a6a7a8a9a0aaabacadaeaffff1f2f3f4f5f6f7acdc",
        (uint8_t *)rid, PSIRP_RID_LEN);

    libpla_io_string_hexread("deadbeef00a0a1a2a3a4a5a6a7a8a9a0aaabacadaeaffff1f2f3f4f5f6f7f8f9",
        (uint8_t *)rid2, PSIRP_RID_LEN);

    libpla_io_string_hexread("1f35a0f6919e0f5c7c44a952c54dc4a28c037070000000000000000000000000",
        (uint8_t *)sid, PSIRP_SID_LEN);

    libpla_io_string_hexread("f716d2bf3fc6e20d941a0b15a20f620b2c2b2f59000000000000000000000000",
        (uint8_t *)sid2, PSIRP_SID_LEN);

    //fprintf(stderr,"Size of the PLA header is: %d\n", (int)sizeof(struct pla_hdr));


	/* Check in loop */
	int i;
	for (i=0; i<COUNT; i++) {
		libpla_pla_header_add(pla_header, NULL, psirp_header, 10, payload, 10);
		
		// This also works if we don't have any data infront of PLA header
		//libpla_pla_header_add(pla_header, NULL, 0, payload, 10);

		/* Try to verify that with pla_receive function */
		memcpy(data, psirp_header, 10);
		memcpy(data+10, pla_header, sizeof(struct pla_hdr));
		memcpy(data+10+sizeof(struct pla_hdr), payload, 10);
		
        /* No rid present in the certificate */
        ret = libpla_pla_receive(data, 10+sizeof(struct pla_hdr)+10, 10, NULL, NULL, 0);

        /* Rid present */
        //ret = libpla_pla_receive(data, 10+sizeof(struct pla_hdr)+10, 10, NULL, rid, 0);

        /* Sid present */
        //ret = libpla_pla_receive(data, 10+sizeof(struct pla_hdr)+10, 10, sid2, NULL, 0);

        /* Rid + Sid present */
        //ret = libpla_pla_receive(data, 10+sizeof(struct pla_hdr)+10, 10, sid, rid, 0);


        //fprintf(stderr, "PLA receive returned: %d\n", ret);
	}

    exit(0);

    /* Test creation of a new certificate */
    pla_cert = libpla_create_certificate("/etc/ttp.cert", "testi.cert", 
      time(NULL), time(NULL)+100000000, 31, 31, (rid_t *)rid);
    libpla_create_certificate("/etc/ttp.cert", "testi2.cert", 
      time(NULL), time(NULL)+1000000, 30, 30, (rid_t *)rid2);

    /* Sign packet with a new certificate */
    libpla_pla_header_add(pla_header, pla_cert, psirp_header, 10, payload, 10);

    /* Try to verify that with pla_receive function */
    memcpy(data, psirp_header, 10);
    memcpy(data+10, pla_header, sizeof(struct pla_hdr));
    memcpy(data+10+sizeof(struct pla_hdr), payload, 10);
    //ret = libpla_pla_receive(data, 10+sizeof(struct pla_hdr)+10, 10, NULL, NULL, 0);
    ret = libpla_pla_receive(data, 10+sizeof(struct pla_hdr)+10, 10, NULL, (rid_t *)rid, 0);

    /* Print certificate information */
    libpla_print_certificates(stdout);

    /* Test search function */
    printf("Testing search...\n");
    pla_cert2 = libpla_search_certificate(NULL, 
        (unsigned char*)pla_cert->pla_header->implicit_certificate);
    libpla_print_certificate_information(stdout, pla_cert2);

    printf("Testing search based on rid:\n");
    libpla_io_hexdump(stdout, (unsigned char *)rid2, PSIRP_RID_LEN);
    printf("\n");
    pla_cert2 = libpla_search_certificate((rid_t *)rid2, NULL);
    libpla_print_certificate_information(stdout, pla_cert2);


    /* Test delete operation */
    printf("Delete returns: %d\n", libpla_delete_certificate((rid_t *)rid2, NULL));
    libpla_delete_certificate((rid_t *)rid, NULL);

    /* Delete initial certificate */
    libpla_get_subject_pk(&pk, &tmp);

    printf("Delete returns: %d\n", libpla_delete_certificate(NULL, pk));

    libpla_print_certificates(stdout);

    free(pla_header);
    free(data);

    PSIRP_DEBUG(PSIRP_DBG_INFO, "Calling libpla_cleanup\n");
    libpla_cleanup();
    psirp_debug_cleanup();

    return 0;
}
